@@ -17,9 +17,11 @@
 #include <string.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <sys/mount.h>
 #include <assert.h>
 
 #include "lib/zt_common.h"
+#include "lib/util_libc.h"
 #include "lib/util_part.h"
 #include "lib/util_path.h"
 #include "boot/s390.h"
@@ -33,6 +35,8 @@
 #include "install.h"
 #include "misc.h"
 
+#define NGDUMP_FSTYPE	"ext4"
+
 /* Pointer to dedicated empty block in bootmap. */
 static disk_blockptr_t empty_block;
 
@@ -219,18 +223,6 @@ add_program_table(int fd, disk_blockptr_t* table, int entries,
 	return rc;
 }
 
-struct component_entry {
-	uint8_t data[23];
-	uint8_t type;
-	component_data compdat;
-} __packed;
-
-typedef enum {
-	component_execute = 0x01,
-	component_load = 0x02,
-	component_signature = 0x03
-} component_type;
-
 static void
 create_component_entry(void* buffer, disk_blockptr_t* pointer,
 		       component_type type, component_data data,
@@ -242,15 +234,15 @@ create_component_entry(void* buffer, disk_blockptr_t* pointer,
 	memset(entry, 0, sizeof(struct component_entry));
 	entry->type = (uint8_t) type;
 	switch (type) {
-		case component_load:
+		case COMPONENT_LOAD:
 			bootmap_store_blockptr(&entry->data, pointer,
 					       info);
 			entry->compdat.load_address = data.load_address;
 			break;
-		case component_execute:
+		case COMPONENT_EXECUTE:
 			entry->compdat.load_psw = data.load_psw;
 			break;
-		case component_signature:
+		case COMPONENT_SIGNATURE:
 			bootmap_store_blockptr(&entry->data, pointer,
 					       info);
 			entry->compdat.sig_head = data.sig_head;
@@ -258,18 +250,6 @@ create_component_entry(void* buffer, disk_blockptr_t* pointer,
 	}
 }
 
-
-struct component_header {
-	uint8_t magic[4];
-	uint8_t type;
-	uint8_t reserved[27];
-}  __packed;
-
-typedef enum {
-	component_header_ipl = 0x00,
-	component_header_dump = 0x01
-} component_header_type;
-
 static void
 create_component_header(void* buffer, component_header_type type)
 {
@@ -347,7 +327,7 @@ add_component_file_range(int fd, const char *filename, struct file_range *reg,
 	rc = add_segment_table(fd, list, count, &segment, info);
 	free(list);
 	if (rc == 0) {
-		create_component_entry(component, &segment, component_load,
+		create_component_entry(component, &segment, COMPONENT_LOAD,
 				       (component_data) load_address, info);
 		/* Return location if requested */
 		if (location != NULL)
@@ -387,7 +367,7 @@ add_component_buffer_align(int fd, void *buffer, size_t size,
 		error_text("Could not write to bootmap file");
 		return -1;
 	}
-	if (type == component_load) {
+	if (type == COMPONENT_LOAD) {
 		/* Fill in component location */
 		loc.addr = data.load_address;
 		loc.size = count * info->phy_block_size;
@@ -433,7 +413,7 @@ add_dummy_buffer(int fd, size_t size, address_t addr, void *component,
 	memset(buffer, 0, size);
 	rc = add_component_buffer(fd, buffer, size,
 				  (component_data) (uint64_t) addr,
-				  component, info, comp_loc, component_load);
+				  component, info, comp_loc, COMPONENT_LOAD);
 	if (rc) {
 		free(buffer);
 		return rc;
@@ -630,7 +610,7 @@ static int add_ipl_program(int fd, char *filename,
 					  (component_data)sig_head,
 					  VOID_ADD(table, offset), info,
 					  &comp_loc[comp_nr],
-					  component_signature);
+					  COMPONENT_SIGNATURE);
 		if (rc) {
 			error_text("Could not add stage3 signature");
 			free(table);
@@ -685,7 +665,7 @@ static int add_ipl_program(int fd, char *filename,
 				  (component_data) (uint64_t)
 				  STAGE3_PARAMS_ADDRESS,
 				  VOID_ADD(table, offset), info,
-				  &comp_loc[comp_nr], component_load);
+				  &comp_loc[comp_nr], COMPONENT_LOAD);
 	free(stage3_params);
 	if (rc) {
 		error_text("Could not add parameters");
@@ -711,7 +691,7 @@ static int add_ipl_program(int fd, char *filename,
 					  (component_data)sig_head,
 					  VOID_ADD(table, offset), info,
 					  &comp_loc[comp_nr],
-					  component_signature);
+					  COMPONENT_SIGNATURE);
 		if (rc) {
 			error_text("Could not add image signature");
 			free(table);
@@ -758,7 +738,7 @@ static int add_ipl_program(int fd, char *filename,
 					  (component_data) ipl->common.parm_addr,
 					  VOID_ADD(table, offset),
 					  info, &comp_loc[comp_nr],
-					  component_load);
+					  COMPONENT_LOAD);
 		if (rc) {
 			error_text("Could not add parmline '%s'",
 				   ipl->common.parmline);
@@ -786,7 +766,7 @@ static int add_ipl_program(int fd, char *filename,
 						  (component_data)sig_head,
 						  VOID_ADD(table, offset), info,
 						  &comp_loc[comp_nr],
-						  component_signature);
+						  COMPONENT_SIGNATURE);
 			if (rc) {
 				error_text("Could not add ramdisk signature");
 				free(table);
@@ -834,7 +814,7 @@ static int add_ipl_program(int fd, char *filename,
 					       (component_data)ipl->envblk_addr,
 					       VOID_ADD(table, offset),
 					       info, &comp_loc[comp_nr],
-					       component_load,
+					       COMPONENT_LOAD,
 					       info->fs_block_size,
 					       &envblk_off);
 			if (rc) {
@@ -877,7 +857,7 @@ static int add_ipl_program(int fd, char *filename,
 		print_components(comp_name, comp_loc, comp_nr);
 	/* Terminate component table */
 	create_component_entry(VOID_ADD(table, offset), NULL,
-			       component_execute,
+			       COMPONENT_EXECUTE,
 			       (component_data) (uint64_t)
 			       (STAGE3_ENTRY | PSW_LOAD),
 			       info);
@@ -930,7 +910,7 @@ add_segment_program(int fd, struct job_segment_data* segment,
 		print_components(comp_name, comp_loc, 1);
 	/* Terminate component table */
 	create_component_entry(VOID_ADD(table, offset), NULL,
-			       component_execute, (component_data) (uint64_t)
+			       COMPONENT_EXECUTE, (component_data) (uint64_t)
 			       PSW_DISABLED_WAIT, info);
 	/* Write component table */
 	rc = disk_write_block_aligned(fd, table, info->phy_block_size,
@@ -988,8 +968,11 @@ get_dump_parmline(char *partition, char *parameters,
 		disk_free_info(info);
 		return -1;
 	}
-	buffer = create_dump_parmline(parameters, "/dev/ram0",
-				      info->partnum, 1);
+	if (is_ngdump_enabled(partition, target))
+		buffer = misc_strdup(parameters);
+	else
+		buffer = create_dump_parmline(parameters, "/dev/ram0",
+					      info->partnum, 1);
 	disk_free_info(info);
 	if (buffer == NULL)
 		return -1;
@@ -1051,9 +1034,9 @@ build_program_table(int fd, char *filename, struct job_data *job,
 			printf("Adding IPL section '%s' (default)\n",
 			       job->name);
 		if (job->data.ipl.is_kdump)
-			component_header = component_header_dump;
+			component_header = COMPONENT_HEADER_DUMP;
 		else
-			component_header = component_header_ipl;
+			component_header = COMPONENT_HEADER_IPL;
 		rc = add_ipl_program(fd, filename,
 				     true, &job->envblk, &job->data.ipl,
 				     &table[0], verbose || job->command_line,
@@ -1068,7 +1051,7 @@ build_program_table(int fd, char *filename, struct job_data *job,
 			       job->name);
 		rc = add_segment_program(fd, &job->data.segment, &table[0],
 					 verbose || job->command_line,
-					 job->add_files, component_header_ipl,
+					 job->add_files, COMPONENT_HEADER_IPL,
 					 info, &job->target);
 		break;
 	case job_dump_partition:
@@ -1080,7 +1063,7 @@ build_program_table(int fd, char *filename, struct job_data *job,
 			       job->name);
 		rc = add_dump_program(fd, &job->data.dump, &table[0],
 					 verbose || job->command_line,
-					 component_header_dump,
+					 COMPONENT_HEADER_DUMP,
 					 info, &job->target);
 		break;
 	case job_menu:
@@ -1089,6 +1072,12 @@ build_program_table(int fd, char *filename, struct job_data *job,
 		for (i=0; i < job->data.menu.num; i++) {
 			switch (job->data.menu.entry[i].id) {
 			case job_ipl:
+				if (job->data.menu.entry[i].data.ipl.common.ignore) {
+					printf("Skipping #%d: IPL section '%s' (missing files)\n",
+					       job->data.menu.entry[i].pos,
+					       job->data.menu.entry[i].name);
+					break;
+				}
 				printf("Adding #%d: IPL section '%s'%s",
 				       job->data.menu.entry[i].pos,
 				       job->data.menu.entry[i].name,
@@ -1097,11 +1086,11 @@ build_program_table(int fd, char *filename, struct job_data *job,
 						" (default)": "");
 				if (job->data.menu.entry[i].data.ipl.is_kdump) {
 					component_header =
-						component_header_dump;
+						COMPONENT_HEADER_DUMP;
 					printf(" (kdump)\n");
 				} else {
 					component_header =
-						component_header_ipl;
+						COMPONENT_HEADER_IPL;
 					printf("\n");
 				}
 				if (job->is_secure != SECURE_BOOT_UNDEFINED)
@@ -1175,23 +1164,163 @@ write_empty_block(int fd, disk_blockptr_t* block, struct disk_info* info)
 }
 
 
-int
-bootmap_create(struct job_data *job, disk_blockptr_t *program_table,
-	       disk_blockptr_t *scsi_dump_sb_blockptr,
-	       disk_blockptr_t **stage1b_list, blocknum_t *stage1b_count,
-	       char **new_device, struct disk_info **new_info)
+static int
+bootmap_install_stages(struct job_data *job, struct disk_info *info, int fd,
+		       disk_blockptr_t **stage1b_list, blocknum_t *stage1b_count)
 {
-	struct scsi_dump_sb scsi_sb;
-	char *device, *filename, *mapname;
 	disk_blockptr_t *stage2_list;
 	blocknum_t stage2_count;
-	struct disk_info *info;
 	size_t stage2_size;
 	void *stage2_data;
-	int fd, rc, part_ext;
+	int rc;
+
+	switch (info->type) {
+	case disk_type_fba:
+		if (boot_get_fba_stage2(&stage2_data, &stage2_size, job))
+			return -1;
+		stage2_count = disk_write_block_buffer(fd, 0, stage2_data,
+						stage2_size, &stage2_list,
+						info);
+		free(stage2_data);
+		if (stage2_count == 0)
+			return -1;
+		rc = install_fba_stage1b(fd, stage1b_list, stage1b_count,
+					 stage2_list, stage2_count, info);
+		free(stage2_list);
+		if (rc)
+			return -1;
+		break;
+	case disk_type_eckd_ldl:
+	case disk_type_eckd_cdl:
+		if (boot_get_eckd_stage2(&stage2_data, &stage2_size, job))
+			return -1;
+		stage2_count = disk_write_block_buffer(fd, 0, stage2_data,
+						stage2_size, &stage2_list,
+						info);
+		free(stage2_data);
+		if (stage2_count == 0)
+			return -1;
+		rc = install_eckd_stage1b(fd, stage1b_list, stage1b_count,
+					  stage2_list, stage2_count, info);
+		free(stage2_list);
+		if (rc)
+			return -1;
+		break;
+	case disk_type_scsi:
+	case disk_type_diag:
+		*stage1b_list = NULL;
+		*stage1b_count = 0;
+		break;
+	}
+	return 0;
+}
+
+
+static int
+bootmap_write_scsi_superblock(int fd, struct disk_info *info,
+			      disk_blockptr_t *scsi_dump_sb_blockptr,
+			      ulong dump_size)
+{
+	struct scsi_dump_sb scsi_sb;
+
+	memset(&scsi_sb, 0, sizeof(scsi_sb));
+	scsi_sb.magic = SCSI_DUMP_SB_MAGIC;
+	scsi_sb.version = 1;
+	scsi_sb.part_start = info->geo.start * info->phy_block_size;
+	scsi_sb.part_size = info->phy_blocks * info->phy_block_size;
+	scsi_sb.dump_offset = 0;
+	scsi_sb.dump_size = dump_size;
+	scsi_sb.csum_offset = 0;
+	scsi_sb.csum_size = SCSI_DUMP_SB_CSUM_SIZE;
+	/* Set seed because otherwise csum over zero block is 0 */
+	scsi_sb.csum = SCSI_DUMP_SB_SEED;
+	return disk_write_block_aligned(fd, &scsi_sb,
+					sizeof(scsi_sb),
+					scsi_dump_sb_blockptr, info);
+}
+
+
+static int
+estimate_scsi_dump_size(struct job_data *job, struct disk_info *info, ulong *dump_size)
+{
+	struct stat st;
+	ulong size;
+
+	/* Use approximated stage 3 size as starting point */
+	size = IMAGE_LOAD_ADDRESS;
+
+	/* Ramdisk */
+	if (job->data.dump.common.ramdisk != NULL) {
+		if (stat(job->data.dump.common.ramdisk, &st))
+			return -1;
+		size += DIV_ROUND_UP(st.st_size, info->phy_block_size);
+		size += 1; /* For ramdisk section entry */
+	}
+	/* Kernel */
+	if (stat(job->data.dump.common.image, &st))
+		return -1;
+	size += DIV_ROUND_UP(st.st_size - IMAGE_LOAD_ADDRESS,
+			     info->phy_block_size);
+	/* Parmfile */
+	size += DIV_ROUND_UP(DUMP_PARAM_MAX_LEN, info->phy_block_size);
+	size += 8;  /* 1x table + 1x script + 3x section + 1x empty
+		       1x header + 1x scsi dump super block */
+	if (size > info->phy_blocks) {
+		error_text("Partition too small for dump tool");
+		return -1;
+	}
+	*dump_size = (info->phy_blocks - size) * info->phy_block_size;
+	return 0;
+}
+
+
+static int
+check_dump_device(const struct job_data *job, const struct disk_info *info,
+		  const char *device)
+{
+	int rc, part_ext;
+
+	/* Check for supported disk and driver types */
+	if ((info->source == source_auto) && (info->type == disk_type_diag)) {
+		error_reason("Unsupported disk type (%s)",
+			     disk_get_type_name(info->type));
+		return -1;
+	}
+	/* Check if secure boot was enabled only for SCSI */
+	if (job->is_secure == SECURE_BOOT_ENABLED &&
+	    info->type != disk_type_scsi) {
+		error_reason("Secure boot forced for non-SCSI disk type");
+		return -1;
+	}
+	rc = util_part_search(device, info->geo.start,
+			      info->phy_blocks, info->phy_block_size, &part_ext);
+	if (rc <= 0 || part_ext) {
+		if (rc == 0)
+			error_reason("No partition");
+		else if (rc < 0)
+			error_reason("Could not read partition table");
+		else if (part_ext)
+			error_reason("Extended partitions not allowed");
+		error_text("Invalid dump device");
+		return -1;
+	}
+	return 0;
+}
+
+
+static int
+bootmap_create_device(struct job_data *job, disk_blockptr_t *program_table,
+		      disk_blockptr_t *scsi_dump_sb_blockptr,
+		      disk_blockptr_t **stage1b_list, blocknum_t *stage1b_count,
+		      char **new_device, struct disk_info **new_info)
+{
+	char *device, *filename;
+	struct disk_info *info;
+	ulong unused_size;
+	int fd;
 
 	/* Get full path of bootmap file */
-	if (job->id == job_dump_partition && !dry_run) {
+	if (!dry_run) {
 		filename = misc_strdup(job->data.dump.device);
 		if (filename == NULL)
 			return -1;
@@ -1214,16 +1343,99 @@ bootmap_create(struct job_data *job, disk_blockptr_t *program_table,
 			goto out_free_filename;
 		}
 	}
+	/* Retrieve target device information */
+	if (disk_get_info(filename, &job->target, &info))
+		goto out_close_fd;
+	if (verbose) {
+		printf("Target device information\n");
+		disk_print_info(info);
+	}
+	if (misc_temp_dev(info->device, 1, &device))
+		goto out_disk_free_info;
+	if (check_dump_device(job, info, device))
+		goto out_misc_free_temp_dev;
+	printf("Building bootmap directly on partition '%s'%s\n",
+	       filename,
+	       job->add_files ? " (files will be added to partition)"
+	       : "");
+	/* For partition dump set raw partition offset
+	   to expected size before end of disk */
+	if (estimate_scsi_dump_size(job, info, &unused_size))
+		goto out_misc_free_temp_dev;
+	if (lseek(fd, unused_size, SEEK_SET) < 0)
+		goto out_misc_free_temp_dev;
+
+	/* Initialize bootmap header */
+	if (bootmap_header_init(fd)) {
+		error_text("Could not init bootmap header at '%s'", filename);
+		goto out_misc_free_temp_dev;
+	}
+	/* Write empty block to be read in place of holes in files */
+	if (write_empty_block(fd, &empty_block, info)) {
+		error_text("Could not write to file '%s'", filename);
+		goto out_misc_free_temp_dev;
+	}
+	/* Build program table */
+	if (build_program_table(fd, filename, job, program_table, info))
+		goto out_misc_free_temp_dev;
+	if (bootmap_write_scsi_superblock(fd, info, scsi_dump_sb_blockptr,
+					  unused_size)) {
+		error_text("Could not write SCSI superblock to file '%s'",
+			   filename);
+		goto out_misc_free_temp_dev;
+	}
+	/* Install stage 2 loader to bootmap if necessary */
+	if (bootmap_install_stages(job, info, fd, stage1b_list, stage1b_count)) {
+		error_text("Could not install loader stages to bootmap");
+		goto out_misc_free_temp_dev;
+	}
+	if (dry_run)
+		misc_free_temp_file(filename);
+	*new_device = device;
+	*new_info = info;
+	close(fd);
+	free(filename);
+	return 0;
+
+out_misc_free_temp_dev:
+	misc_free_temp_dev(device);
+out_disk_free_info:
+	disk_free_info(info);
+out_close_fd:
+	close(fd);
+out_free_filename:
+	free(filename);
+	return -1;
+}
+
+
+static int
+bootmap_create_file(struct job_data *job, char *bootmap_dir,
+		    disk_blockptr_t *program_table,
+		    disk_blockptr_t *scsi_dump_sb_blockptr,
+		    disk_blockptr_t **stage1b_list, blocknum_t *stage1b_count,
+		    char **new_device, struct disk_info **new_info)
+{
+	char *device, *filename, *mapname;
+	struct disk_info *info;
+	int fd;
+
+	/* Get full path of bootmap file */
+	filename = misc_make_path(bootmap_dir, BOOTMAP_TEMPLATE_FILENAME);
+	if (filename == NULL)
+		return -1;
+	/* Create temporary bootmap file */
+	fd = mkstemp(filename);
+	if (fd == -1) {
+		error_reason(strerror(errno));
+		error_text("Could not create file '%s':", filename);
+		goto out_free_filename;
+	}
 	/* Retrieve target device information. Note that we have to
 	 * call disk_get_info_from_file() to also get the file system
 	 * block size. */
-	if (job->id == job_dump_partition) {
-		if (disk_get_info(filename, &job->target, &info))
-			goto out_close_fd;
-	} else {
-		if (disk_get_info_from_file(filename, &job->target, &info))
-			goto out_close_fd;
-	}
+	if (disk_get_info_from_file(filename, &job->target, &info))
+		goto out_close_fd;
 	/* Check for supported disk and driver types */
 	if ((info->source == source_auto) && (info->type == disk_type_diag)) {
 		error_reason("Unsupported disk type (%s)",
@@ -1247,63 +1459,10 @@ bootmap_create(struct job_data *job, disk_blockptr_t *program_table,
 		if (check_menu_positions(&job->data.menu, job->name, info))
 			goto out_misc_free_temp_dev;
 	}
-	if (job->id == job_dump_partition) {
-		rc = util_part_search(device, info->geo.start, info->phy_blocks,
-				      info->phy_block_size, &part_ext);
-		if (rc <= 0 || part_ext) {
-			if (rc == 0)
-				error_reason("No partition");
-			else if (rc < 0)
-				error_reason("Could not read partition table");
-			else if (part_ext)
-				error_reason("Extended partitions not allowed");
-			error_text("Invalid dump device");
-			goto out_misc_free_temp_dev;
-		}
-		printf("Building bootmap directly on partition '%s'%s\n",
-		       filename,
-		       job->add_files ? " (files will be added to partition)"
-		       : "");
-	} else {
-		printf("Building bootmap in '%s'%s\n", job->target.bootmap_dir,
-		       job->add_files ? " (files will be added to bootmap file)"
-		       : "");
-	}
-	/* For partition dump set raw partition offset
-	   to expected size before end of disk */
-	if (job->id == job_dump_partition) {
-		struct stat st;
-		ulong size;
-		ulong unused_size;
 
-		/* Use approximated stage 3 size as starting point */
-		size = IMAGE_LOAD_ADDRESS;
-
-		/* Ramdisk */
-		if (job->data.dump.common.ramdisk != NULL) {
-			if (stat(job->data.dump.common.ramdisk, &st))
-				goto out_misc_free_temp_dev;
-			size += DIV_ROUND_UP(st.st_size, info->phy_block_size);
-			size += 1; /* For ramdisk section entry */
-		}
-		/* Kernel */
-		if (stat(job->data.dump.common.image, &st))
-			goto out_misc_free_temp_dev;
-		size += DIV_ROUND_UP(st.st_size - IMAGE_LOAD_ADDRESS,
-				     info->phy_block_size);
-		/* Parmfile */
-		size += DIV_ROUND_UP(DUMP_PARAM_MAX_LEN, info->phy_block_size);
-		size += 8;  /* 1x table + 1x script + 3x section + 1x empty
-			       1x header + 1x scsi dump super block */
-		if (size > info->phy_blocks) {
-			error_text("Partition too small for dump tool");
-			goto out_misc_free_temp_dev;
-		}
-		unused_size = (info->phy_blocks - size) * info->phy_block_size;
-		if (lseek(fd, unused_size, SEEK_SET) < 0)
-			goto out_misc_free_temp_dev;
-		scsi_sb.dump_size = unused_size;
-	}
+	printf("Building bootmap in '%s'%s\n", bootmap_dir,
+	       job->add_files ? " (files will be added to bootmap file)"
+	       : "");
 
 	/* Initialize bootmap header */
 	if (bootmap_header_init(fd)) {
@@ -1318,69 +1477,20 @@ bootmap_create(struct job_data *job, disk_blockptr_t *program_table,
 	/* Build program table */
 	if (build_program_table(fd, filename, job, program_table, info))
 		goto out_misc_free_temp_dev;
-	if (job->id == job_dump_partition) {
-		scsi_sb.magic = SCSI_DUMP_SB_MAGIC;
-		scsi_sb.version = 1;
-		scsi_sb.part_start = info->geo.start * info->phy_block_size;
-		scsi_sb.part_size = info->phy_blocks * info->phy_block_size;
-		scsi_sb.dump_offset = 0;
-		scsi_sb.csum_offset = 0;
-		scsi_sb.csum_size = SCSI_DUMP_SB_CSUM_SIZE;
-		/* Set seed because otherwise csum over zero block is 0 */
-		scsi_sb.csum = SCSI_DUMP_SB_SEED;
-		disk_write_block_aligned(fd, &scsi_sb,
-					 sizeof(scsi_sb),
-					 scsi_dump_sb_blockptr, info);
-	} else
-		scsi_dump_sb_blockptr->linear.block = 0;
 
-	/* Add stage 2 loader to bootmap if necessary */
-	switch (info->type) {
-	case disk_type_fba:
-		if (boot_get_fba_stage2(&stage2_data, &stage2_size, job))
-			goto out_misc_free_temp_dev;
-		stage2_count = disk_write_block_buffer(fd, 0, stage2_data,
-						stage2_size, &stage2_list,
-						info);
-		free(stage2_data);
-		if (stage2_count == 0) {
-			error_text("Could not write to file '%s'", filename);
-			goto out_misc_free_temp_dev;
-		}
-		if (install_fba_stage1b(fd, stage1b_list, stage1b_count,
-					stage2_list, stage2_count, info))
-			goto out_misc_free_temp_dev;
-		free(stage2_list);
-		break;
-	case disk_type_eckd_ldl:
-	case disk_type_eckd_cdl:
-		if (boot_get_eckd_stage2(&stage2_data, &stage2_size, job))
-			goto out_misc_free_temp_dev;
-		stage2_count = disk_write_block_buffer(fd, 0, stage2_data,
-						stage2_size, &stage2_list,
-						info);
-		free(stage2_data);
-		if (stage2_count == 0) {
-			error_text("Could not write to file '%s'", filename);
-			goto out_misc_free_temp_dev;
-		}
-		if (install_eckd_stage1b(fd, stage1b_list, stage1b_count,
-					 stage2_list, stage2_count, info))
-			goto out_misc_free_temp_dev;
-		free(stage2_list);
-		break;
-	case disk_type_scsi:
-	case disk_type_diag:
-		*stage1b_list = NULL;
-		*stage1b_count = 0;
-		break;
+	scsi_dump_sb_blockptr->linear.block = 0;
+
+	/* Install stage 2 loader to bootmap if necessary */
+	if (bootmap_install_stages(job, info, fd, stage1b_list, stage1b_count)) {
+		error_text("Could not install loader stages to file '%s'",
+			   filename);
+		goto out_misc_free_temp_dev;
 	}
 	if (dry_run) {
 		misc_free_temp_file(filename);
-	} else if (job->id != job_dump_partition) {
+	} else {
 		/* Rename to final bootmap name */
-		mapname = misc_make_path(job->target.bootmap_dir,
-				BOOTMAP_FILENAME);
+		mapname = misc_make_path(bootmap_dir, BOOTMAP_FILENAME);
 		if (mapname == NULL)
 			goto out_misc_free_temp_dev;
 		if (rename(filename, mapname)) {
@@ -1403,9 +1513,146 @@ out_disk_free_info:
 	disk_free_info(info);
 out_close_fd:
 	close(fd);
-	if (job->id != job_dump_partition)
-		misc_free_temp_file(filename);
+	misc_free_temp_file(filename);
 out_free_filename:
 	free(filename);
 	return -1;
 }
+
+
+static int
+ngdump_create_meta(const char *path)
+{
+	char *filename = NULL;
+	FILE *fp;
+	int rc;
+
+	util_asprintf(&filename, "%s/ngdump.meta", path);
+
+	fp = fopen(filename, "w");
+	if (!fp) {
+		free(filename);
+		error_reason(strerror(errno));
+		error_text("Could not create file '%s'", filename);
+		return -1;
+	}
+	free(filename);
+
+	rc = fprintf(fp, "version=1\n");
+	if (rc < 0)
+		return -1;
+	rc = fprintf(fp, "file=\n");
+	if (rc < 0)
+		return -1;
+	rc = fprintf(fp, "sha256sum=\n");
+	if (rc < 0)
+		return -1;
+	rc = fclose(fp);
+	if (rc < 0)
+		return -1;
+
+	return 0;
+}
+
+
+static int
+bootmap_create_device_ngdump(struct job_data *job, disk_blockptr_t *program_table,
+			     disk_blockptr_t *scsi_dump_sb_blockptr,
+			     disk_blockptr_t **stage1b_list, blocknum_t *stage1b_count,
+			     char **new_device, struct disk_info **new_info)
+{
+	char mount_point[] = "/tmp/zipl-dump-mount-point-XXXXXX";
+	struct disk_info *info;
+	char *device;
+	int rc;
+
+	/* Retrieve target device information */
+	if (disk_get_info(job->data.dump.device, &job->target, &info))
+		return -1;
+	if (misc_temp_dev(info->device, 1, &device))
+		goto out_disk_free_info;
+	if (check_dump_device(job, info, device))
+		goto out_misc_free_temp_dev;
+	/* Create a mount point directory */
+	if (mkdtemp(mount_point) == NULL) {
+		error_reason(strerror(errno));
+		error_text("Could not create mount point '%s'", mount_point);
+		goto out_misc_free_temp_dev;
+	}
+	if (!dry_run) {
+		char *cmd = NULL;
+		util_asprintf(&cmd, "mkfs.%s -qF %s >/dev/null",
+			      NGDUMP_FSTYPE, job->data.dump.device);
+		if (verbose)
+			printf("Formatting partition '%s'\n",
+				job->data.dump.device);
+		rc = system(cmd);
+		free(cmd);
+		if (rc) {
+			error_reason(strerror(errno));
+			error_text("Could not format partition '%s':",
+				   job->data.dump.device);
+			goto out_rmdir;
+		}
+	}
+	/*
+	 * Mount partition where bootmap file and also a dump file will
+	 * be stored.
+	 */
+	if (mount(job->data.dump.device, mount_point, NGDUMP_FSTYPE, 0, NULL)) {
+		error_reason(strerror(errno));
+		error_text("Could not mount partition '%s':",
+			   job->data.dump.device);
+		goto out_rmdir;
+	}
+	if (bootmap_create_file(job, mount_point, program_table,
+				scsi_dump_sb_blockptr,
+				stage1b_list, stage1b_count,
+				new_device, new_info))
+		goto out_umount;
+	if (ngdump_create_meta(mount_point))
+		goto out_umount;
+	/* Cleanup */
+	umount(mount_point);
+	rmdir(mount_point);
+	misc_free_temp_dev(device);
+	disk_free_info(info);
+	return 0;
+
+out_umount:
+	umount(mount_point);
+out_rmdir:
+	rmdir(mount_point);
+out_misc_free_temp_dev:
+	misc_free_temp_dev(device);
+out_disk_free_info:
+	disk_free_info(info);
+	return -1;
+}
+
+
+int
+bootmap_create(struct job_data *job, disk_blockptr_t *program_table,
+	       disk_blockptr_t *scsi_dump_sb_blockptr,
+	       disk_blockptr_t **stage1b_list, blocknum_t *stage1b_count,
+	       char **new_device, struct disk_info **new_info)
+{
+	if (job->id == job_dump_partition) {
+		if (is_ngdump_enabled(job->data.dump.device, &job->target))
+			return bootmap_create_device_ngdump(job, program_table,
+							    scsi_dump_sb_blockptr,
+							    stage1b_list,
+							    stage1b_count,
+							    new_device, new_info);
+		else
+			return bootmap_create_device(job, program_table,
+						     scsi_dump_sb_blockptr,
+						     stage1b_list, stage1b_count,
+						     new_device, new_info);
+	} else
+		return bootmap_create_file(job, job->target.bootmap_dir,
+					   program_table,
+					   scsi_dump_sb_blockptr,
+					   stage1b_list, stage1b_count,
+					   new_device, new_info);
+}
