@@ -30,6 +30,8 @@
 #include "zipl.h"
 #include "envblk.h"
 
+const char *blsdir;
+
 /* Command line options */
 static struct option options[] = {
 	{ "config",		required_argument,	NULL, 'c'},
@@ -758,7 +760,8 @@ static void error_text_section(const char *text, const char *section, const char
 
 
 static int
-check_common_ipl_data(struct job_common_ipl_data *common, const char *section)
+check_common_ipl_data(struct job_common_ipl_data *common, const char *section,
+		      bool may_ignore)
 {
 	uint64_t max_parm_size, len;
 	char *buffer = NULL;
@@ -768,10 +771,16 @@ check_common_ipl_data(struct job_common_ipl_data *common, const char *section)
 	if (common->image != NULL) {
 		rc = misc_read_file(common->image, &buffer, &size, 0);
 		if (rc) {
-			error_text_section("Image file", section, common->image);
+			if (may_ignore && common->optional) {
+				printf("Optional section '%s': Missing image file '%s'\n",
+				       section, common->image);
+				error_clear_reason();
+				common->ignore = true;
+				rc = 0;
+				goto skip_image;
+			}
 			return rc;
 		}
-
 		if (size < MAX_COMMAND_LINE_SIZE + sizeof(uint64_t)) {
 			error_text_section("Image file", section, common->image);
 			return -1;
@@ -781,7 +790,7 @@ check_common_ipl_data(struct job_common_ipl_data *common, const char *section)
 		if (!max_parm_size)
 			max_parm_size = LEGACY_MAXIMUM_PARMLINE_SIZE;
 
-		len = strlen(common->parmline);
+		len = common->parmline ? strlen(common->parmline) : 0;
 		if (len > max_parm_size) {
 			error_text("The length of the parameters line "
 				   "(%d bytes) exceeds the allowed maximum "
@@ -789,12 +798,19 @@ check_common_ipl_data(struct job_common_ipl_data *common, const char *section)
 			return -1;
 		}
 	}
-
+skip_image:
 	if (common->ramdisk != NULL) {
 		rc = misc_check_readable_file(common->ramdisk);
 		if (rc) {
-			error_text_section("Ramdisk file", section, common->ramdisk);
-			return rc;
+			if (common->optional) {
+				printf("Optional section '%s': Missing ramdisk file '%s'\n",
+				       section, common->ramdisk);
+				error_clear_reason();
+				common->ignore = true;
+				rc = 0;
+			} else {
+				return rc;
+			}
 		}
 	}
 	return 0;
@@ -802,13 +818,17 @@ check_common_ipl_data(struct job_common_ipl_data *common, const char *section)
 
 static int
 check_job_ipl_data(struct job_ipl_data *ipl, char *name,
-		   struct job_envblk_data *envblk)
+		   struct job_envblk_data *envblk, bool may_ignore)
 {
 	int rc;
 
-	rc = check_common_ipl_data(&ipl->common, name);
+	rc = check_common_ipl_data(&ipl->common, name, may_ignore);
 	if (rc)
 		return rc;
+
+	if (ipl->common.ignore)
+		return 0;
+
 	return finalize_ipl_address_data(ipl, name, envblk);
 }
 
@@ -876,6 +896,92 @@ check_job_dump_images(struct job_dump_data* dump, char* name)
 }
 
 
+int
+check_job_images_ngdump(struct job_dump_data* dump, char *name)
+{
+	const char *helper = TOOLS_LIBDIR "/zipl_helper.prepare-ngdump";
+	char *ppn_cmd = NULL;
+	char *line = NULL;
+	FILE *fp;
+	int rc;
+
+	misc_asprintf(&ppn_cmd, "%s %s", helper, dump->device);
+	printf("Run %s\n", ppn_cmd);
+
+	fp = popen(ppn_cmd, "r");
+	if (fp == NULL) {
+		error_reason("Failed to run popen(%s,\"r\",)");
+		free(ppn_cmd);
+		return -1;
+	}
+	free(ppn_cmd);
+
+	while (fscanf(fp, "%m[^\n]\n", &line) != EOF) {
+		char *param = NULL;
+		char *value = NULL;
+
+		int n = sscanf(line, "%m[^=]=%m[^\n]\n", &param, &value);
+
+		if (n != 2) {
+			if (verbose)
+				printf("%s\n", line);
+			free(line);
+			free(param);
+			free(value);
+			continue;
+		}
+
+		if (strcmp(param, "kernel") == 0) {
+			dump->common.image = value;
+			value = NULL;
+		} else if (strcmp(param, "initrd") == 0) {
+			dump->common.ramdisk = value;
+			value = NULL;
+		} else if (strcmp(param, "cmdline") == 0) {
+			dump->common.parmline = value;
+			value = NULL;
+		} else {
+			printf("%s\n", line);
+		}
+
+		free(line);
+		free(param);
+		free(value);
+	}
+
+	switch (pclose(fp)) {
+	case 0:
+		/* success */
+		break;
+	case -1:
+		error_reason("Failed to run pclose");
+		return -1;
+	default:
+		error_reason("Script could not determine dump parameters");
+		return -1;
+	}
+
+	rc = misc_check_readable_file(dump->common.image);
+	if (rc) {
+		error_text("Need external file '%s' for dump",
+			   dump->common.image);
+		return rc;
+	}
+	dump->common.image_addr = IMAGE_LOAD_ADDRESS;
+
+	rc = misc_check_readable_file(dump->common.ramdisk);
+	if (rc) {
+		error_text("Need external file '%s' for dump",
+			   dump->common.ramdisk);
+		return rc;
+	}
+	dump->common.ramdisk_addr = UNSPECIFIED_ADDRESS;
+
+	dump->common.parm_addr = UNSPECIFIED_ADDRESS;
+	return finalize_common_address_data(&dump->common, name);
+}
+
+
 static int
 check_job_menu_data(struct job_menu_data *menu, struct job_envblk_data *envblk)
 {
@@ -887,9 +993,15 @@ check_job_menu_data(struct job_menu_data *menu, struct job_envblk_data *envblk)
 		case job_ipl:
 			rc = check_job_ipl_data(&menu->entry[i].data.ipl,
 						menu->entry[i].name,
-						envblk);
+						envblk, true);
 			if (rc)
 				return rc;
+			/* default_pos is 1-indexed */
+			if (menu->default_pos == i + 1 &&
+			    menu->entry[i].data.ipl.common.ignore) {
+				error_text("Cannot ignore default entry");
+				return -1;
+			}
 			break;
 		case job_print_usage:
 		case job_print_version:
@@ -917,7 +1029,7 @@ check_job_ipl_tape_data(struct job_ipl_tape_data *ipl, char* name)
 			return rc;
 		}
 	}
-	rc = check_common_ipl_data(&ipl->common, name);
+	rc = check_common_ipl_data(&ipl->common, name, false);
 	if (rc)
 		return rc;
 	return finalize_common_address_data(&ipl->common, name);
@@ -1016,7 +1128,7 @@ check_job_data(struct job_data* job)
 		break;
 	case job_ipl:
 		rc = check_job_ipl_data(&job->data.ipl, job->name,
-					&job->envblk);
+					&job->envblk, false);
 		break;
 	case job_menu:
 		rc = check_job_menu_data(&job->data.menu, &job->envblk);
@@ -1295,6 +1407,12 @@ get_job_from_section_data(char* data[], struct job_data* job, char* section)
 			if (rc)
 				return rc;
 		}
+		job->data.ipl.common.optional = false;
+		if (data[(int) scan_keyword_optional] != NULL) {
+			job->data.ipl.common.optional =
+				atoi(data[(int) scan_keyword_optional]) == 1;
+		}
+		job->data.ipl.common.ignore = false;
 		break;
 	case section_ipl_tape:
 		/* Tape IPL job */
@@ -1702,7 +1820,6 @@ get_job_from_config_file(struct command_line* cmdline, struct job_data* job)
 	struct scan_token* scan;
 	struct scan_token* new_scan;
 	const char *filename = NULL;
-	char *blsdir;
 	char* source;
 	int i, rc, scan_size;
 
@@ -1742,7 +1859,7 @@ get_job_from_config_file(struct command_line* cmdline, struct job_data* job)
 	} else {
 		blsdir = ZIPL_DEFAULT_BLSDIR;
 	}
-	rc = scan_bls(blsdir, &scan, scan_size);
+	rc = scan_bls(&scan, scan_size);
 	if (rc) {
 		error_text("BLS parsing '%s'", blsdir);
 		return rc;
@@ -1892,3 +2009,9 @@ job_get(int argc, char* argv[], struct job_data** data)
 	*data = job;
 	return rc;
 }
+
+bool
+is_ngdump_enabled(const char* device, struct job_target_data* target)
+{
+	return disk_is_nvme(device, target);
+}
